@@ -209,7 +209,7 @@ gdk_window_impl_quartz_begin_paint_region (GdkPaintable *paintable,
       x_offset = y_offset = 0;
 
       window = GDK_WINDOW (drawable_impl->wrapper);
-      while (window && ((GdkWindowObject *) window)->bg_pixmap == GDK_PARENT_RELATIVE_BG)
+      while (window && bg_pixmap == GDK_PARENT_RELATIVE_BG)
         {
           /* If this window should have the same background as the parent,
            * fetch the parent. (And if the same goes for the parent, fetch
@@ -218,6 +218,16 @@ gdk_window_impl_quartz_begin_paint_region (GdkPaintable *paintable,
           x_offset += ((GdkWindowObject *) window)->x;
           y_offset += ((GdkWindowObject *) window)->y;
           window = GDK_WINDOW (((GdkWindowObject *) window)->parent);
+          bg_pixmap = ((GdkWindowObject *) window)->bg_pixmap;
+        }
+
+      if (bg_pixmap == NULL || bg_pixmap == GDK_NO_BG || bg_pixmap == GDK_PARENT_RELATIVE_BG)
+        {
+          /* Parent relative background but the parent doesn't have a
+           * pixmap.
+           */ 
+          g_free (rects);
+          return;
         }
 
       /* Note: There should be a CG API to draw tiled images, we might
@@ -824,6 +834,7 @@ _gdk_windowing_window_init (void)
 {
   GdkWindowObject *private;
   GdkWindowImplQuartz *impl;
+  GdkDrawableImplQuartz *drawable_impl;
   NSRect rect;
 
   g_assert (_gdk_root == NULL);
@@ -841,6 +852,12 @@ _gdk_windowing_window_init (void)
   private->state = 0; /* We don't want GDK_WINDOW_STATE_WITHDRAWN here */
   private->window_type = GDK_WINDOW_ROOT;
   private->depth = 24;
+
+  drawable_impl = GDK_DRAWABLE_IMPL_QUARTZ (private->impl);
+  
+  drawable_impl->wrapper = GDK_DRAWABLE (private);
+  drawable_impl->colormap = gdk_screen_get_system_colormap (_gdk_screen);
+  g_object_ref (drawable_impl->colormap);
 }
 
 void
@@ -851,6 +868,7 @@ _gdk_windowing_window_destroy (GdkWindow *window,
   GdkWindowObject *private;
   GdkWindowImplQuartz *impl;
   GdkWindowObject *parent;
+  GdkWindow *mouse_window;
 
   private = GDK_WINDOW_OBJECT (window);
   impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
@@ -878,24 +896,21 @@ _gdk_windowing_window_destroy (GdkWindow *window,
   if (window == _gdk_quartz_keyboard_grab_window)
     gdk_keyboard_ungrab (0);
 
+  _gdk_quartz_drawable_finish (GDK_DRAWABLE (impl));
+
+  mouse_window = _gdk_quartz_events_get_mouse_window (FALSE);
+  if (window == mouse_window ||
+      _gdk_quartz_window_is_ancestor (window, mouse_window))
+    _gdk_quartz_events_update_mouse_window (_gdk_root);
+
   if (!recursing && !foreign_destroy)
     {
-      GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
-      GdkWindow *mouse_window;
-
-      mouse_window = _gdk_quartz_events_get_mouse_window (FALSE);
-      if (window == mouse_window ||
-          _gdk_quartz_window_is_ancestor (window, mouse_window))
-        _gdk_quartz_events_update_mouse_window (_gdk_root);
-
       GDK_QUARTZ_ALLOC_POOL;
 
-      _gdk_quartz_drawable_finish (GDK_DRAWABLE (impl));
-
       if (impl->toplevel)
 	[impl->toplevel close];
       else if (impl->view)
-	[impl->view release];
+	[impl->view removeFromSuperview];
 
       GDK_QUARTZ_RELEASE_POOL;
     }
@@ -1145,10 +1160,10 @@ move_resize_window_internal (GdkWindow *window,
 
   impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
 
-  if ((x == private->x) &&
-      (y == private->y) &&
-      (width == impl->width) &&
-      (height == impl->height))
+  if ((x == -1 || (x == private->x)) &&
+      (y == -1 || (y == private->y)) &&
+      (width == -1 || (width == impl->width)) &&
+      (height == -1 || (height == impl->height)))
     {
       return;
     }
@@ -2867,9 +2882,6 @@ gdk_window_configure_finished (GdkWindow *window)
 void
 gdk_window_destroy_notify (GdkWindow *window)
 {
-  /* FIXME: Implement. We should call this from -[GdkQuartzWindow dealloc] or
-   * -[GdkQuartzView dealloc], although I suspect that currently they leak
-   * anyway. */
 }
 
 void 
